/*
  Rhino test framework

   Copyright (C) 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.RhinoTests;

import java.lang.reflect.Method;



/**
 * This is test engine which contains basic methods which are used to run the
 * particular test suites. This class also contains various assertion-like
 * methods (at this moment, only a few assertion types are needed for testing).
 *
 * @author Pavel Tisnovsky
 */
public abstract class BaseRhinoTest {
    /**
     * Number of tests that passed.
     */
    private int passed = 0;

    /**
     * Number of tests that failed.
     */
    private int failed = 0;

    /**
     * Set up the test suite.
     * 
     * @param args
     *            arguments which are passed throught command line
     */
    protected abstract void setUp(String[] args);

    /**
     * Tear down the test suite.
     */
    protected abstract void tearDown();

    /**
     * Run all tests in the test suite.
     * 
     * @param args
     *            arguments which are passed throught command line
     */
    protected void doTests(String[] args) {
        long t1 = System.currentTimeMillis();

        // setup test
        setUp(args);
        try {
            runAllTests();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        finally {
            // tear down test
            tearDown();
            // check duration
            long t2 = System.currentTimeMillis();
            // print all results
            printResults(t1, t2, this.passed, this.failed);
        }
    }

    /**
     * Run all tests in the test suite.
     * 
     * @throws Exception
     *             this exception can be thrown by any test.
     */
    private void runAllTests() throws Exception {
        // first we get all methods declared in a class
        Method[] methods = getClass().getDeclaredMethods();
        // then we check, if this method is a proper test method
        for (Method method : methods) {
            tryToInvokeTestMethod(method);
        }
    }

    /**
     * Call a method if its name begins with "test" prefix. Such methods should
     * have public or protected access declaration.
     * 
     * @param method
     *            method which is called
     * @throws Exception
     *             this exception can be thrown by any test.
     */
    private void tryToInvokeTestMethod(Method method) throws Exception {
        String methodName = method.getName();
        // all test methods should have a prefix "test"
        if (methodName.startsWith("test")) {
            try {
                method.invoke(this);
                printTestPassed(methodName);
                this.passed++;
            }
            catch (Exception e) {
                printTestFailed(methodName, e);
                this.failed++;
                e.printStackTrace();
            }
        }
    }

    /**
     * Print information about test which passed.
     * 
     * @param methodName
     *            name of method implementing the test.
     */
    private void printTestPassed(String methodName) {
        log("PASSED", '.', methodName);
    }

    /**
     * Print information about test which failed.
     * 
     * @param methodName
     *            name of method implementing the test.
     */
    private void printTestFailed(String methodName, Exception e) {
        log("FAILED", '.', methodName + ": FAILED " + e.getMessage());
    }

    /**
     * Print summary about all tests in one test suite.
     * 
     * @param t1
     *            time when the test suite was started
     * @param t2
     *            time when the test suite finishes
     * @param passedTests
     *            number of passed tests
     * @param failedTests
     *            number of failed tests
     */
    private void printResults(long t1, long t2, int passedTests, int failedTests) {
        log("SUMMARY", ' ', "total: " + (passedTests + failedTests) + "  passed: " + passedTests + "  failed: " + failedTests + "  duration: " + (t2-t1) + " ms");
    }

    /**
     * Log into a standard output.
     * 
     * @param prefix
     *            string printed before class name.
     * @param delimiter
     *            delimiter between prefix and the class name.
     * @param methodName
     *            name of method implementing the test.
     */
    protected void log(String prefix, char delimiter, String methodName) {
        System.out.println(prefix + ": " + this.getClass().getName() + delimiter + methodName);
    }

    /**
     * Log into a standard output.
     * 
     * @param prefix
     *            string printed before class name.
     * @param delimiter
     *            delimiter between prefix and the class name.
     * @param methodName
     *            name of method implementing the test.
     * @param exception
     *            message generated from exception
     */
    protected void log(String prefix, char delimiter, String methodName, String exception) {
        System.out.println(prefix + ": " + this.getClass().getName() + delimiter + methodName + " " + exception);
    }

    /**
     * Assertion if two objects are equal.
     * 
     * @param o1
     *            first object
     * @param o2
     *            second object
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertEquals(Object o1, Object o2, String message) throws AssertionError {
        assertTrue(o1.equals(o2), message);
    }

    /**
     * Assertion if two characters are equal.
     * 
     * @param value1
     *            first value to be compared
     * @param value2
     *            second value to be compared
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertEquals(char value1, char value2, String message) throws AssertionError {
        assertTrue(value1 == value2, message);
    }

    /**
     * Assertion if two bytes are equal.
     * 
     * @param value1
     *            first value to be compared
     * @param value2
     *            second value to be compared
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertEquals(byte value1, byte value2, String message) throws AssertionError {
        assertTrue(value1 == value2, message);
    }

    /**
     * Assertion if two shorts are equal.
     * 
     * @param value1
     *            first value to be compared
     * @param value2
     *            second value to be compared
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertEquals(short value1, short value2, String message) throws AssertionError {
        assertTrue(value1 == value2, message);
    }

    /**
     * Assertion if two integers are equal.
     * 
     * @param value1
     *            first value to be compared
     * @param value2
     *            second value to be compared
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertEquals(int value1, int value2, String message) throws AssertionError {
        assertTrue(value1 == value2, message);
    }

    /**
     * Assertion if two longs are equal.
     * 
     * @param value1
     *            first value to be compared
     * @param value2
     *            second value to be compared
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertEquals(long value1, long value2, String message) throws AssertionError {
        assertTrue(value1 == value2, message);
    }

    /**
     * Assertion if two floats are equal.
     * 
     * @param value1
     *            first value to be compared
     * @param value2
     *            second value to be compared
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertEquals(float value1, float value2, String message) throws AssertionError {
        assertTrue(value1 == value2, message);
    }

    /**
     * Assertion if two doubles are equal.
     * 
     * @param value1
     *            first value to be compared
     * @param value2
     *            second value to be compared
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertEquals(double value1, double value2, String message) throws AssertionError {
        assertTrue(value1 == value2, message);
    }

    /**
     * Assertion for true condition.
     * 
     * @param bool
     *            boolean value to be tested
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertTrue(boolean bool, String message) throws AssertionError {
        if (!bool) {
            throw new AssertionError(message);
        }
    }

    /**
     * Assertion for true condition.
     * 
     * @param bool
     *            boolean value to be tested
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertTrue(Boolean bool, String message) throws AssertionError {
        if (!bool.booleanValue()) {
            throw new AssertionError(message);
        }
    }

    /**
     * Assertion for false condition.
     * 
     * @param bool
     *            boolean value to be tested
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertFalse(boolean bool, String message) throws AssertionError {
        if (bool) {
            throw new AssertionError(message);
        }
    }

    /**
     * Assertion for false condition.
     * 
     * @param bool
     *            boolean value to be tested
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertFalse(Boolean bool, String message) throws AssertionError {
        if (bool.booleanValue()) {
            throw new AssertionError(message);
        }
    }

    /**
     * Assertion if given reference is null.
     * 
     * @param object
     *            reference to be tested
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertNull(Object object, String message) throws AssertionError {
        assertTrue(object == null, message);
    }

    /**
     * Assertion if given reference is not null.
     * 
     * @param object
     *            reference to be tested
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertNotNull(Object object, String message) throws AssertionError {
        assertTrue(object != null, message);
    }

    /**
     * Assertion if given string is empty.
     * 
     * @param string
     *            string to be tested.
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertEmpty(String str, String message) throws AssertionError {
        assertTrue(str.isEmpty(), message);
    }

    /**
     * Assertion if given string is not empty.
     * 
     * @param string
     *            string to be tested.
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertNotEmpty(String str, String message) throws AssertionError {
        assertFalse(str.isEmpty(), message);
    }

    /**
     * Assertion if object is an instance of a given class.
     * 
     * @param object
     *            object reference to be tested
     * @param clazz
     *            class
     * @param message
     *            message printed when assertion has failed
     * @throws AssertionError
     *             thrown to indicate that an assertion has failed
     */
    protected static void assertType(Object object, Class<?> clazz, String message) throws AssertionError {
        assertTrue(object.getClass().equals(clazz), message);
    }

    /**
     * Returns version of Java. The input could have the following form: "1.7.0_06"
     * and we are interested only in "7" in this case.
     * 
     * @return Java version
     */
    protected int getJavaVersion() {
        String javaVersionStr = System.getProperty("java.version");
        String[] parts = javaVersionStr.split("\\.");
        return Integer.parseInt(parts[1]);
    }
}
